/*globals define*/
/*jslint unparam: true */

define(['testsuite/qunit', 'jquery', 'ep/fn/sendasform', 'jquery/cookie', '$ready!'], function (QUnit, $, ep) {

    QUnit.module("ep/fn/sendasform", {

        setup: function () {
            $.cookie('SecToken', '1337');
        },

        teardown: function () {
            $.removeCookie('SecToken');
        }
    });

    QUnit.test("calling method to send element as form", function () {
        var apply = $('<div id="Apply" />').appendTo('#qunit-fixture'),
            link = $('<button type="button" class="Button CancelButton ep-js" name="CancelEdit" id="cancelEdit">Cancel</button>').appendTo('#qunit-fixture'),
            formOptions = {
                'ViewAction': 'View',
                'ViewObjectID': '123',
                'ChangeAction': 'CancelEdit',
                'ChangeObjectID': '456'
            },
            _key,
            form,
            elem;

        // call sendAsForm method with all available options
        link.sendAsForm({
            formOptions: formOptions,
            additional: {
                fn: function () {
                    $.cookie('SecToken', '1337');

                    if ($('#Apply').length) {
                        $('#Apply').addClass('Disabled');
                    }
                },
                body: "#qunit-fixture",
                target: "newWindow",
                send: false
            }
        });

        // this element is used to test the functionality of the addional function handed over in additional.fn
        QUnit.equal(
            /Disabled/.test(apply[0].className),
            false,
            'Element should not contain class "Disabled"'
        );

        // trigger click on button to insert form
        link.trigger('click');

        QUnit.equal(
            /Disabled/.test(apply[0].className),
            true,
            'Element should contain class "Disabled" set by the addional function handed over in additional.fn'
        );

        QUnit.equal(
            $('#qunit-fixture').find('form').length,
            1,
            'Should contain a form'
        );

        QUnit.equal(
            $('#qunit-fixture').find('form').attr('target'),
            "newWindow",
            'Should contain the target attribute "newWindow"'
        );

        form = $('#qunit-fixture').find('form').eq(0);

        QUnit.equal(
            form.attr('method').toLowerCase(),
            'post',
            'Should contain a form with method "post"'
        );

        // check hidden inputs needed as form parameters
        for (_key in formOptions) {
            if (formOptions.hasOwnProperty(_key)) {
                elem = form.find('input[name="' + _key + '"]');

                QUnit.equal(
                    elem.length,
                    1,
                    'Should contain one element with name "' + _key + '"'
                );

                QUnit.equal(
                    elem.val(),
                    formOptions[_key],
                    'Element "' + _key + '" should contain value "' + formOptions[_key] + '"'
                );
            }
        }

        // check security token
        elem = form.find('input[name="SecToken"]');

        QUnit.equal(
            elem.length,
            1,
            'Should contain one element with name "SecToken"'
        );

        QUnit.equal(
            elem.val(),
            '1337',
            'Element "SecToken" should contain value "1337"'
        );

    });

});
