define(['testsuite/qunit', 'jquery', 'ep/inject-sec-token', 'jquery/cookie'], function (QUnit, $, ep) {

    QUnit.module("ep/inject-sec-token", {

        setup: function () {
            postForm = $('<form method="post" />').appendTo('#qunit-fixture');
            defaultForm = $('<form class="default" />').appendTo('#qunit-fixture');
        },

        teardown: function () {
            postForm.empty();
            defaultForm.empty();
        }
    });

    QUnit.test("calling without options, using defaults", function () {
        $.cookie('SecToken', '1337');
        ep.injectSecToken();

        QUnit.equal(
            postForm.find('input[name=SecToken]').val(),
            '1337',
            'Should add an input field with correct security token'
        );

        QUnit.equal(
            defaultForm.find('input').length,
            0,
            'Should not add an input in a form with method "get"'
        );

        $.removeCookie('SecToken');
    });

    QUnit.test("Calling with customized options", function () {
       $.cookie('myCustomizedToken', '42');
        ep.injectSecToken({
            'tokenName': 'myCustomizedToken',
            'selector': 'form.default'
        });

        QUnit.equal(
            defaultForm.find('input[name=myCustomizedToken]').val(),
            '42',
            'Should add an input field with correct security token'
        );

        $.removeCookie('myCustomizedToken');
    });

    QUnit.test("Test link with ChangeAction", function () {
        var changeActionLink = $('<a href="?ChangeAction=Delete">link</a>').appendTo('#qunit-fixture');

        $.cookie('SecToken', '987654321');
        ep.injectSecToken();

        console.log($('#qunit-fixture').html());
        console.log(/SecToken=987654321/.test(changeActionLink.attr('href')));

        QUnit.equal(
            /SecToken=987654321/.test(changeActionLink.attr('href')),
            true,
            'Should contain a parameter with security token'
        );

        $.removeCookie('SecToken');
    });

});
