use strict;

use Test::More tests => 12;
use Readonly;

use DE_EPAGES::Core::API::Error qw ( GetError ExistsError );
use DE_EPAGES::Presentation::API::Object::CSRFPredator;

Readonly my $SERVER_SECRET => 'Store4711';
Readonly my $USER_SECRET   => '10.10.10.10Mozilla/5.0 (Windows NT 6.1; WOW64; rv:29.0) Gecko/20100101 Firefox/29.0';

ConstructorTest();
MethodTest();

sub ConstructorTest {
    eval { DE_EPAGES::Presentation::API::Object::CSRFPredator->new(); };
    ok( ExistsError(), 'constructor checks (1)' );
    is( GetError->code, 'PARAMETER_MISSING', 'constructor checks (2)' );

    eval { DE_EPAGES::Presentation::API::Object::CSRFPredator->new($USER_SECRET); };
    ok( ExistsError(), 'constructor checks (3)' );
    is( GetError->code, 'PARAMETER_MISSING', 'constructor checks (4)' );

    eval { DE_EPAGES::Presentation::API::Object::CSRFPredator->new($USER_SECRET, $SERVER_SECRET); };
    ok( !ExistsError(), 'constructor checks (5)' ) or note( GetError->longMessage);

    my $Predator = DE_EPAGES::Presentation::API::Object::CSRFPredator->new(
        $USER_SECRET,
        $SERVER_SECRET
    );
    isa_ok( $Predator, 'DE_EPAGES::Presentation::API::Object::CSRFPredator' );
}

sub MethodTest {
    my $Predator = _GetPredator();
    my $SecToken = $Predator->generateSecurityToken();
    like( $SecToken, qr{\S+_[0-9]{10,}}, 'generateSecurityToken() - looks like a valid token' );
    ok( $Predator->validateSecurityToken($SecToken), 'generateSecurityToken() - is valid' );

    eval { $Predator->validateSecurityToken( 'otto' ); };
    ok( ExistsError(), 'validateSecurityToken() - invalid input produces error' );
    is( GetError->code, 'INVALID_CSRF_TOKEN', 'validateSecurityToken() - invalid input produces nice error code' );

    eval { $Predator->validateSecurityToken(); };
    ok( ExistsError(), 'validateSecurityToken() - no input produces error' );
    is( GetError->code, 'INVALID_CSRF_TOKEN', 'validateSecurityToken() - no input produces nice error code' );
}


sub _GetPredator {
    return DE_EPAGES::Presentation::API::Object::CSRFPredator->new(
        $USER_SECRET,
        $SERVER_SECRET
    );
}
