#!/bin/env perl

use strict;
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Config qw( StoreType GetBusinessUnits StoreTypesWithCartridge );
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle RunOnStore);


RunScript(
    'Sub' => \&Main
);

sub Main {
    local $| = 1;

    my $BU;
    push @{$BU->{StoreType($_)}}, $_ foreach @{GetBusinessUnits({conf => 1})};

    my $Cartridge = 'DE_EPAGES::Permission';
    foreach my $Store (@{_StoresWithCartridge($Cartridge, $BU)}) {
        RunOnStore(
            Store => $Store,
            Sub => sub  {
               my $dbi = GetCurrentDBHandle();
               if($dbi->driverName() eq 'mysql'){
                   #changes for file Permission/Database/mysql/Tables/changeaction.sql
                   unless ($dbi->existsTableColumn('changeaction', 'usesectokencookie')) {
                       print "patch DBStructure for Cartridge $Cartridge in $Store\n";
                       GetLog()->info('add column changeaction.usesectokencookie after column changeaction.changeactionid');
                       $dbi->addTableColumnAfterColumn('changeaction', 'changeactionid', 'usesectokencookie bool  not null default 1');
                       $dbi->modifyTableColumnDropDefault('changeaction','usesectokencookie');
                   }
               }
            }, );
    }
}

sub _StoresWithCartridge ($$) {
    my ($Cartridge, $BU) = @_;

    my @Stores;
    foreach my $StoreType (@{StoreTypesWithCartridge("$Cartridge")}) {
        push @Stores, $_ foreach @{$BU->{$StoreType}};
    }
    return \@Stores;
}

1;

